#include "inputShaderHelper.h"

#ifdef AdobeRGB1998
    constant float4 profileToRGB_R = float4(1.398283,-0.398283064, 0.0,0.0  );
    constant float4 profileToRGB_G = float4(0.0,1.0,0.0,0.0  );
    constant float4 profileToRGB_B = float4(0.0, -0.0429382771,1.04293835,0.0  );
#endif

#ifdef DISPLAY_P3
    constant float4 profileToRGB_R = float4(1.2249,-0.2247, 0.0,0.0  );
    constant float4 profileToRGB_G = float4(-0.0420,1.0419,0.0,0.0  );
    constant float4 profileToRGB_B = float4(-0.0197, -0.0786,1.0979,0.0  );
#endif

#ifdef BT2020
    // values obtained from:
    // Report ITU-R BT.2407-0 (10/2017) - Colour gamut conversion from Recommendation ITU-R BT.2020 to Recommendation ITU-R BT.709
    constant float4 profileToRGB_R = float4( 1.6605, -0.5876, -0.0728, 0.0);
    constant float4 profileToRGB_G = float4(-0.1246,  1.1329, -0.0083, 0.0);
    constant float4 profileToRGB_B = float4(-0.0182, -0.1006,  1.1187, 0.0);
#endif

#ifdef FULL_RANGE
//full range values from wikipedia https://en.wikipedia.org/wiki/YCbCr#ITU-R_BT.709_conversion
    constant float4 yuv2R = float4( 1.402,1., 0.0,0.0  );
    constant float4 yuv2G = float4(-0.714136,1.,-0.344136,0.0  );
    constant float4 yuv2B = float4( 0.000000,1., 1.772,0.0  );
    constant float4 stdbias = float4(-0.5  , 0.0,  -0.5   ,0.0);
#else
    constant float4 yuv2R = float4( 1.793,1.164383, 0.000000,0.0);
    constant float4 yuv2G = float4(-0.534,1.164383,-0.213,0.0);
    constant float4 yuv2B = float4( 0.000000,1.164383, 2.115,0.0);
    constant float4 stdbias = float4(-0.5  , -0.0625,  -0.5   ,0.0);
#endif



float3 lessThan(float3 a,float3 b)
{
    float3 retVal=float3(0.0);
    
    if(a.x<b.x)
        retVal.x=1.0;
    if(a.y<b.y)
        retVal.y=1.0;
    if(a.z<b.z)
        retVal.z=1.0;
    return retVal;
}

float4 toGamma(float4 linearRGB)
{
    float3 cutoff = lessThan(linearRGB.rgb, float3(0.0031308));
    float3 higher = float3(1.055)*pow(linearRGB.rgb, float3(1.0/2.4)) - float3(0.055);
    float3 lower = linearRGB.rgb * float3(12.92);

    return float4(mix(higher, lower, cutoff), linearRGB.a);
}

float4 toLinear(float4 sRGB)
{
    float3 cutoff = lessThan(sRGB.rgb, float3(0.04045));
    float3 higher = pow((sRGB.rgb + float3(0.055))/float3(1.055), float3(2.4));
    float3 lower = sRGB.rgb/float3(12.92);

    return float4(mix(higher, lower, cutoff), sRGB.a);
}


fragment float4 fragmentFunc(inputVertexType in [[stage_in]]
                                 ,texture2d<half> inputTex0 [[ texture(0) ]]
                                 ,texture2d<half> inputTex1 [[ texture(1) ]]
                                 ,texture2d<half> inputTex2 [[ texture(2) ]]
                                 ,texture2d<half> inputTex3 [[ texture(3) ]]
                                 ,texture2d<half> maskTex [[ texture(4) ]]
                                 ,constant planarUniforms& uniforms[[ buffer(0) ]]
)
{
    vec4 yuvIn;
    yuvIn.a=0.0;
    yuvIn.g = textureFetchNearest(inputTex0, in.texCoordY).r;
    yuvIn.br = textureFetchNearest(inputTex1, in.texCoordUV).rg;
    
#ifdef HAS_ALPHA
    yuvIn.a = textureFetchNearest(inputTex2, in.texCoordY).r;
#endif

#ifdef IS_10BIT
    yuvIn*=float4(64.0, 64.0, 64.0, 64.0);;
#endif
    
#ifdef IS_12BIT
    yuvIn*=float4(16.0, 16.0, 16.0, 16.0);
#endif
    
    yuvIn+=stdbias;

#ifndef HAS_ALPHA
    yuvIn.a=1.0;
#endif
    
    
    float4 outColor=vec4(dot(yuv2R, yuvIn),
                  dot(yuv2G, yuvIn),
                  dot(yuv2B, yuvIn),
                  yuvIn.a);
    

#ifdef USE_COLOR_PROFILE
    float4 linear=toLinear(outColor);

    float4 srgbLinear=float4(dot(profileToRGB_R, linear),
                    dot(profileToRGB_G, linear),
                    dot(profileToRGB_B, linear),
                    outColor.a);

    outColor=toGamma(srgbLinear);
#endif

#ifdef PREMULT
    outColor.rgb*=outColor.a;
#endif

#ifdef USE_MASK
    outColor.a*=textureFetchNearest(maskTex,in.maskCoord).a;
#endif

#ifdef USE_END_COLOR
    outColor=mix(uniforms.endColor,outColor,uniforms.endOpacity);
#endif

    return outColor;
}
